@@ -13,6 +13,8 @@ use Carp;
 use Scalar::Util qw(looks_like_number);
 use URI::Escape;
 use Text::ParseWords;
+use JSON;
+
 our $AUTOLOAD;
 
 =head1 NAME
@@ -64,7 +66,6 @@ sub get_requests {
     my $jmx = shift;
     my $args = shift;
 
-    my $request;
     my $do_read = $self->attribute || $self->value;
     if ($self->alias) {
         my $alias = JMX::Jmx4Perl::Alias->by_name($self->alias);
@@ -72,11 +73,17 @@ sub get_requests {
         $do_read = $alias->type eq "attribute";
     }
     my @requests = ();
+    my $request;
     if ($do_read) {
-        push @requests,JMX::Jmx4Perl::Request->new(READ,$self->_prepare_read_args($jmx));
+        $request = JMX::Jmx4Perl::Request->new(READ,$self->_prepare_read_args($jmx));
     } else {
-        push @requests,JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
+        $request = JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
     }
+    my $method = $self->{np}->opts->{method} || $self->{config}->{method};
+    if ($method) {
+        $request->method($method);
+    }
+    push @requests,$request;
 
     if ($self->base) {
         if (!looks_like_number($self->base)) {
@@ -120,7 +127,7 @@ sub extract_responses {
     my $self = shift;    
     my $responses = shift;
     my $requests = shift;
-    my $target = shift;
+    my $opts = shift || {};
     my $np = $self->{np};
 
     # Get response/request pair
@@ -136,15 +143,14 @@ sub extract_responses {
     if (defined($delta)) {
         $value = $self->_delta_value($request,$resp,$delta);
         unless (defined($value)) {
-            push @extra_requests,$self->_switch_on_history($request,$target);
+            push @extra_requests,$self->_switch_on_history($request,$opts->{target});
             $value = 0;
         }
     }
     
     # Normalize value 
     my ($value_conv,$unit) = $self->_normalize_value($value);
-    # Common args
-    my $label = "'".$self->_get_name(cleanup => 1)."'";
+    my $label = $self->_get_name(cleanup => 1);
     if ($self->base) {
         # Calc relative value 
         my $base_value = $self->_base_value($self->base,$responses,$requests);
@@ -156,22 +162,31 @@ sub extract_responses {
                           critical => $critical,warning => $warning,
                           min => 0,max => $base_value,
                           $self->unit ? (uom => $self->unit) : ());
-        
         # Do the real check.
         my ($code,$mode) = $self->_check_threshhold($rel_value);
+        # For Multichecks, we remember the label of a currently failed check
+        if ($code != OK && $opts->{error_stat}) {
+            my $label = $self->{config}->{key} || $self->{config}->{name};
+            if ($label) {
+                my $arr = $opts->{error_stat}->{$code} || [];
+                push @$arr,$label;
+                $opts->{error_stat}->{$code} = $arr;
+            }
+        }
         my ($base_conv,$base_unit) = $self->_normalize_value($base_value);
         $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,rel_value => $rel_value, 
                                                     value => $value_conv, unit => $unit,base => $base_conv, 
-                                                    base_unit => $base_unit));            
+                                                    base_unit => $base_unit, prefix => $opts->{prefix}));            
     } else {
         # Performance data
         $np->add_perfdata(label => $label,
-                          critical => $self->critical, warning => $self->warning,
+                          critical => $self->critical, warning => $self->warning, 
                           value => $value,$self->unit ? (uom => $self->unit) : ());
         
         # Do the real check.
         my ($code,$mode) = $self->_check_threshhold($value);
-        $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,value => $value_conv, unit => $unit));                    
+        $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,value => $value_conv, unit => $unit,
+                                                    prefix => $opts->{prefix}));                    
     }
     return @extra_requests;
 }
@@ -193,7 +208,9 @@ sub _null_safe_value {
     my $self = shift;
     my $value = shift;
     if (defined($value)) {
-        if (ref($value) && $self->string) {
+        if (JSON::is_bool($value)) {
+            return "$value";
+        } elsif (ref($value) && $self->string) {
             # We can deal with complex values withing string comparison
             if (ref($value) eq "ARRAY") {
                 return join ",",@{$value};
@@ -236,6 +253,11 @@ sub _delta_value {
         } else {
             $hist_val = $history;
         }
+        if (!@$hist_val) {
+            # Can happen in some scenarios when requesting the first history entry,
+            # we are return 0 here
+            return 0;
+        }
         my $old_value = $hist_val->[0]->{value};
         my $old_time = $hist_val->[0]->{timestamp};
         my $value = $self->_extract_value($req,$resp);
@@ -279,12 +301,13 @@ sub _base_value {
 # Normalize value if a unit-of-measurement is given.
 
 # Units and how to convert from one level to the next
-my @UNITS = ([ qw(us ms s m h d) ],[qw(B KB MB GB TB)]);
+my @UNITS = ([ qw(ns us ms s m h d) ],[qw(B KB MB GB TB)]);
 my %UNITS = 
   (
+   ns => 1,   
    us => 10**3,
    ms => 10**3,
-   s => 1,
+   s => 10**3,
    m => 60,
    h => 60,
    d => 24,
@@ -321,7 +344,7 @@ sub _normalize_value {
                 # Go down the scale ...
                 return ($value,$unit) if $i == 0;
                 for my $j (reverse(0 .. $i-1)) {
-                    if ($ret <= 1) {     
+                    if ($ret < 1) {     
                         $ret *= $UNITS{$units->[$j+1]};
                         $u = $units->[$j];
                     } else {
@@ -336,16 +359,17 @@ sub _normalize_value {
     die "Unknown unit '$unit' for value $value";
 }
 
-
-
 sub _verify_response {
     my ($self,$req,$resp) = @_;
     my $np = $self->{np};
     if ($resp->is_error) {
-        $np->nagios_die("Error: ".$resp->status." ".$resp->error_text."\nStacktrace:\n".$resp->stacktrace);
+        my $stacktrace = $resp->stacktrace;
+        my $extra = "";
+        $extra = ref($stacktrace) eq "ARRAY" ? join "\n",@$stacktrace : $stacktrace if $stacktrace;
+        $np->nagios_die("Error: ".$resp->status." ".$resp->error_text.$extra);
     }
     
-    if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string)) { 
+    if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string) && !JSON::is_bool($resp->value)) { 
         $np->nagios_die("Response value is a " . ref($resp->value) .
                         ", not a plain value. Did you forget a --path parameter ?". " Value: " . 
                         Dumper($resp->value));
@@ -360,18 +384,29 @@ sub _get_name {
         if ($self->name) {
             $name = $self->name;
         } else {
-            # Default name
-            $name = $self->alias ? 
-          "[".$self->alias.($self->path ? "," . $self->path : "") ."]" : 
-            $self->value ? 
-              "[" . $self->value . "]" :
-            "[".$self->mbean.",".$self->attribute.($self->path ? "," . $self->path : "")."]";
+            # Default name, tried to be generated from various parts
+            if ($self->alias) {
+                $name = "[".$self->alias.($self->path ? "," . $self->path : "") ."]";
+            } else {
+                my $val = $self->value;
+                if ($val) {
+                    $name = "[" . $self->value . "]";
+                } else {
+                    my $a_or_o = $self->attribute || $self->operation || "";
+                    my $p = $self->path ? "," . $self->path : "";
+                    $name = "[" . $self->mbean . "," . $a_or_o . $p . "]";
+                }
+            }
         }
     }
     if ($args->{cleanup}) {
         # Enable this when '=' gets forbidden
         $name =~ s/=/#/g;
     }
+    # Prepare label for usage with Nagios::Plugin, which will blindly 
+    # add quotes if a space is contained in the label.
+    # We are doing the escape of quotes ourself here
+    $name =~ s/'/''/g;
     return $name;
 }
 
@@ -399,8 +434,21 @@ sub _prepare_exec_args {
     my $np = $self->{np};
     my $jmx = shift;
 
-    my @args = @_;
-
+    # Merge CLI arguments and arguments from the configuration,
+    # with CLI arguments taking precedence
+    my @cli_args = @_;
+    my $config_args = $self->{config}->{args};
+    my @args = ();
+    if ($config_args) {
+        my @c_args = (@$config_args);
+        while (@cli_args || @c_args) {
+            my $cli_arg = shift @cli_args;
+            my $config_arg = shift @c_args;
+            push @args, defined($cli_arg) ? $cli_arg : $config_arg;
+        }
+    } else {
+        @args = @cli_args;
+    }
     if ($self->alias) {
         my @req_args = $jmx->resolve_alias($self->alias);
         $np->nagios_die("Cannot resolve operation alias ",$self->alias()) unless @req_args >= 2;
@@ -413,11 +461,19 @@ sub _prepare_exec_args {
 sub _split_attr_spec {
     my $self = shift;
     my $name = shift;
-
-    return &parse_line('/',0,$name);
+    my @ret = ();
+    # Text:ParseWords is used for split on "/" taking into account
+    # quoting and escaping
+    for my $p (&parse_line("/",1,$name)) {
+        # We need to 'unescape' things ourselves
+        # since we want quotes to remain in the names (using '0'
+        # above would kill those quotes, too). 
+        $p =~ s|\\(.)|$1|sg;
+        push @ret,$p;
+    }    
+    return @ret;
 }
 
-
 sub _check_threshhold {
     my $self = shift;
     my $value = shift;
@@ -564,7 +620,13 @@ sub _format_label {
             $ret .= $p;
         }
     }
-    return $ret;
+    if ($args->{prefix}) {
+        my $prefix = $args->{prefix};
+        $prefix =~ s/\%c/$STATUS_TEXT{$args->{code}}/g;
+        return  $prefix . $ret;
+    } else {
+        return $ret;
+    }
 }
 
 sub _format_char {
@@ -590,7 +652,7 @@ my $CHECK_CONFIG_KEYS = {
                          "label" => "label",
                          # New:
                          "value" => "value",
-                         "null" => "null"
+                         "null" => "null",
                         };
 
 
